from optparse import OptionParser
import datetime
import glob
import os
import array 

def Log(filename,string):
    String = str(datetime.datetime.utcnow()) +"  "+ string + "\n"
    try:
        AppendFile(filename,String)
    except:
        print "Error: Unable to write to log. Log string on next line."
        print String
    return datetime.datetime.utcnow()

def AppendFile(filename,string):
    fp1 = file(filename,'a')
    fp1.write(string)
    fp1.close()

def OpenRAWImage(filename,format):

    Image = array.array('H') # unsigned short 2bytes
    fp1 = open(filename, 'rb')
    fp1.seek( 0, 2)
    ImageLength = fp1.tell() /2
    fp1.seek( 0)
    Image.fromfile(fp1,ImageLength)
    fp1.close()
    Image.byteswap( ) 
    return Image.tolist( ) 

def InterleaveList3D (d1, d2, d3):
    Length = len(d1)
    OutList = [0]*Length*3
    i = 0
    while (i < Length):
        j = (i*3)
        OutList[j] = d1[i]
        OutList[j+1] = d2[i]
        OutList[j+2] = d3[i]
        i += 1
    return OutList

def AppendTransform (d1,d2,d3, coff, devider):
    Length = len(d1)
    i = 0
    while (i < Length):
        X = d1[i]* coff[0] + d2[i]* coff[1] + d3[i]* coff[2]
        Y = d1[i]* coff[3] + d2[i]* coff[4] + d3[i]* coff[5]
        Z = d1[i]* coff[6] + d2[i]* coff[7] + d3[i]* coff[8]
        d1[i] = X / devider
        d2[i] = Y / devider
        d3[i] = Z / devider
        i += 1
    return (d1,d2,d3)

def ReduceBitCount (data, inbits, outbits):

    Devider = 2**(inbits-outbits)
    UpperLimit = 2**outbits-1
    LoverLimit = 0
    Length = len(data)
    i = 0
    while (i < Length):
        data[i] = int(data[i] / Devider)
        if data[i] > UpperLimit:
            data[i] = UpperLimit
        if data[i] < LoverLimit:
            data[i] = LoverLimit
        i += 1

    return data	

def Main ():
    parser = OptionParser()
    parser.add_option("-i", "--input", dest="Input" )
    parser.add_option("-o", "--output", dest="Output")
    parser.add_option("-l", "--log", action="store_true",  dest="Log_True" )
    parser.add_option("-f", "--logfile", dest="LogFile")
    parser.add_option("-p", "--parameter", dest="Param" )
    (options, args) = parser.parse_args()

    if (options.Param):
        Parameter = options.Param
    else:
        Parameter = ""

    if (options.LogFile):
        LogFile = options.LogFile
    else:
        LogFile = "log.txt"

    if (options.Log_True):
        String = "Info: Script start"
        Log(LogFile,String)
        String = 'Info: Using param "' + Parameter + '"'
        Log(LogFile,String)


    if (options.Input):
        if (options.Log_True):
            String = 'Info: Running Glob with "' + options.Input + '"'
            Log(LogFile,String)
        List = glob.glob(options.Input)
        for InnFilename in List:
            if (options.Output):
                OutFilename = options.Output + RemovePath(RemoveExtension(InnFilename)) + ".tif"
            else:
                OutFilename = RemoveExtension(InnFilename) + ".tif"
            if os.path.exists(OutFilename):
                continue

            #  kdu_expand -i 00000295.j2k -o im1.raw,im2.raw,im3.raw -raw_components
            ExecString  = "kdu_expand"
            ExecString += " -i " + InnFilename
            ExecString += " -o im1.raw,im2.raw,im3.raw"
            ExecString += " -raw_components"
            ExecString += " " + Parameter
            os.system(ExecString)

            if (options.Log_True):
                String = "Info: Decompresed JPEG2000 file " + InnFilename
                Log(LogFile,String)
    
            Raw12bitToPPM("im1.raw", "im2.raw", "im3.raw", "im.ppm")

            if (options.Log_True):
                String = "Info: Transformed XYZ to RGB, aotput as PPM"
                Log(LogFile,String)

            # C:\Program Files\IrfanView\i_view32.exe c:\test.bmp  /tifc=1 /convert=c:\test.jpg
            ExecString  = 'i_view32.exe'
            ExecString += " im.ppm"
            ExecString += " /tifc=1"
            ExecString += " /convert=" + OutFilename
            os.system(ExecString)

            if (options.Log_True):
                String = "Info: Converted PPM to TIFF and output to file " + OutFilename
                Log(LogFile,String)

            os.remove("im.ppm")
            os.remove("im1.raw")
            os.remove("im2.raw")
            os.remove("im3.raw")

            if (options.Log_True):
                String = "Info: Temporary files deleted "
                Log(LogFile,String)


def Raw12bitToPPM(xFile, yFile, zFile, ppmFile):
	
    Im1 = OpenRAWImage(xFile,"RAW16")
    Im2 = OpenRAWImage(yFile,"RAW16")
    Im3 = OpenRAWImage(zFile,"RAW16")
    # Linear transformation from RGB space to XYZ space
    #(Im1,Im2,Im3) = AppendTransform (Im1,Im2,Im3, [490, 310, 200, 176.97, 812.40, 10.63, 0, 10, 990], 176.97)
    # Linear transformation from XYZ space to RGB space
    (Im1,Im2,Im3) = AppendTransform (Im1,Im2,Im3, [2.36461, -0.89654, -0.46807, -0.51517, 1.42641, 0.08876, 0.0052, -0.01441, 1.0092], 1)
    Im  = InterleaveList3D (Im1, Im2, Im3)

    Im  = ReduceBitCount (Im,12,8)
 
    rgbdata = array.array('B',Im)
    
    RGB888toPPM(rgbdata, ppmFile, 2048, 1080, "Created by Python")


def RGB888toPPM(rgbdata, filename, width, height, comment):
    Buf  = "P6\x0A"
    Buf += "# " + comment + "\x0A"
    Buf += str(width) +" "+ str(height) + "\x0A"
    Buf += "255\x0A"

    fp1 = file(filename, 'wb')
    fp1.write(Buf)
    rgbdata.tofile(fp1)
    fp1.close()



        
def RemoveExtension(string):
    i = string.rfind('.')
    return string[:i]

def RemovePath(string):
    i = string.rfind('\\')
    j = string.rfind('/')
    if j < i:
        return string[(i+1):]
    else:
        return string[(j+1):]
    
Main()

        
